<?php

/**
 * @file 
 * This file is used to resize
 */
class ImageFactoryResizeImage {

  private $path;
  private $output;
  public $c1 = 255; // int 0-255 Red
  public $c2 = 255; // int 0-255 G
  public $c3 = 255; // int 0-255 B
  private $alpha = FALSE; // transparency
  private $useAlpha = FALSE; //transparent image

  public function getOutput() {
    return $this->output;
  }

  public function setOutput($output) {
    $this->output = $output;
  }

  public function getPath() {
    return $this->path;
  }

  public function setPath($path) {
    $this->path = $path;
  }

  public function getAlpha() {
    return $this->alpha;
  }

  public function setAlpha($alpha) {
    if ($alpha < 128 && $alpha > -1)
      $this->alpha = $alpha;
  }

  public function closeUseAlpha() {
    $this->useAlpha = FALSE;
  }

  public function openUseAlpha() {
    $gdinfo = gd_info();
    preg_match('|[\d\.]+|', $gdinfo['GD Version'], $match);
    if (!count($match) || intval($match[0]) < 2) {
      throw new Exception('Function  imagecolorallocatealpha need gd (2.0)');
    }
    $this->useAlpha = TRUE;
  }

  /**
   * construct
   * 
   * @param string $path output directory
   * @param string $output image file name(without extention name)
   */
  public function __construct($path = 'image/', $output = FALSE) {

    if (substr($path, -1) != '/')
      $path .='/';

    if (!function_exists('gd_info')) {
      throw new Exception('Miss GD library');
    }

    $this->setPath($path);
    if (!$output)
      $this->setOutput(time());
  }

  /**
   * Resize image.
   * 
   * @param string $image_form
   *   Source image  link resource.
   * @param int $width_to
   *   Destination width. 
   * @param int $height_to
   *   Destination height. 
   * @param int $top
   *   X-coordinate of source point. 
   * @param int $left
   *   Y-coordinate of source point. 
   * @param string $error
   *   The message if error.
   * 
   * @return string|boolean
   *   Resize success or fail.
   * 
   * @throws Exception
   */
  public function reseizeImage($imageFrom, $widthTo, $heightTo, $left, $top, &$result = '') {
    try {
      $target = $imageFrom;

      if (!file_exists($target)) {
        throw new Exception('Without file:'.$target);
      }
      // if file not image ,getimagesize() will  return FALSE and throw E_WARNING  
      $imageInfo = getimagesize($target);
      if (!$imageInfo) {
        throw new Exception('Type of image illegal');
      }
      list($width, $height, $type) = $imageInfo;

      $imFrom = false;

      switch ($type) {
        case 1:
          $imFrom = @imagecreatefromgif($target);
          break;
        case 2:
          $imFrom = @imagecreatefromjpeg($target);
          break;
        case 3:
          //set backgroun transparent
          $this->openUseAlpha();
          $this->setAlpha(127);
          $imFrom = @imagecreatefrompng($target);
          break;
        default:
          throw new Exception('wrong type image');
          break;
      }

      if (!$imFrom) {
        throw new Exception('Init image resource fail.');
      }
      $imTo = imagecreatetruecolor($widthTo, $heightTo);
      if (!$this->alpha) {
        $color = imagecolorallocate($imTo, $this->c1, $this->c2, $this->c3);
      }
      else {
        //save image alpha info
        imagesavealpha($imTo, true);
        $color = imagecolorallocatealpha($imTo, $this->c1, $this->c2, $this->c3, $this->getAlpha());
      }
      imagefill($imTo, 0, 0, $color);

      $copyResult = imagecopyresampled($imTo, $imFrom, 0, 0, $left, $top, $widthTo, $heightTo, $widthTo, $heightTo);


      if (!$copyResult) {
        throw new Exception('Copy image fail');
      }

      $return = false;
      $fileOutput = $this->getPath() . $this->getOutput() . '.';

      switch ($type) {
        case 1:
          $fileOutput .= 'gif';
          $return = imagegif($imTo, $fileOutput);
          break;
        case 2:
          $fileOutput .= 'jpg';
          $return = imagejpeg($imTo, $fileOutput);
          break;
        case 3:
          $fileOutput .= 'png';
          $return = imagepng($imTo, $fileOutput);
          break;
        default:
          throw new Exception('wrong type image');
          break;
      }

      imagedestroy($imFrom);
      imagedestroy($imTo);

      $result = $fileOutput;
      return $result;
    } catch (Exception $e) {
      $result = $e->getMessage();
      return FALSE;
    }
  }

}